package PureMessage::BATV;

use strict;

use Digest::SHA1;

our $VERSION = '5.509';

sub hash {
    my ($key, $salt, $address) = @_;
    my $hash = Digest::SHA1->new();
    (my $local = $address) =~ s/@.*$//;
    $hash->add($key);
    $hash->add($salt);
    $hash->add(lc $local);
    return substr($hash->hexdigest, 0, 6);
}

sub days {
    my $now = time;
    return ($now / 86400) % 1000;
}

sub new {
    my ($this, $config) = @_;
    my $self = bless {}, ref $this || $this;
    if ($config) {
        for my $item (split /\s*,\s*/, $config) {
            my ($key, $value) = ($item =~ m/^(.*?)\s*=>\s*(.*)$/);
            $self->{$key} = $value;
        }
    }

    return $self;
}

sub max_age {
    my ($self, $max) = @_;
    if (defined $max) {
        my $old = $self->{max_age};
        $self->{max_age} = $max;
        return $old;
    }
    return $self->{max_age};
}

sub get_config {
    my ($self) = @_;

    my @items = ();
    for my $key (keys %$self) {
        push @items, "$key => $self->{$key}";
    }

    return join(', ', @items);
}

sub strip_watermark {
    my ($self, $address) = @_;
    $address =~ s/^prvs=\d{4}[0-9a-f]{6}=//i;
    return $address;
}

sub watermark {
    my ($self, $address, $days) = @_;

    $days = sprintf("%03d", $days ? $days : days());
    my $key_number = $self->{current};
    die "No key data available" unless defined $key_number;
    my $key = $self->{"key$key_number"};
    die "No key" unless defined $key;

    return '' if $address eq '';

    # strip away any existing batv watermark
    $address = $self->strip_watermark($address);

    my $hash = hash($key, $days, $address);
    return "prvs=${key_number}${days}${hash}=$address";
}

sub age {
    my ($days) = @_;
    my $current = days;
    $current += 1000 if $current < $days;
    return $current - $days;
}

sub is_out_of_date {
    my ($self, $days, $max_age) = @_;
    $max_age ||= $self->{max_age};
    return 1 unless age($days) <= $max_age;
    return 0;
}

sub verify {
    my ($self, $address, $max_age) = @_;

    if ($address =~ m/^prvs=(\d)(\d{3})([0-9a-f]{6})=(.*)$/i) {
        my ($key_number, $days, $hash, $original_address) = ($1, $2, $3, $4);
        $max_age ||= $self->{max_age};
        my $key = $self->{"key$key_number"} or return undef;
        return undef if $self->is_out_of_date($days, $max_age);
        my $expected = lc hash($key, $days, $original_address);
        return undef unless lc($hash) eq $expected;
        return $original_address;
    }
    return undef;
}

my $base64 = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz/+";

=item generate

Generate a key suitable for use with BATV.
=cut
sub generate {
    my ($self, $index) = @_;

    my $key = '';
    for (my $i = 0 ; $i < 32 ; $i++) {
        $key .= substr($base64, int(rand() * 64), 1);
    }
    if (!$index) {
        $index = (($self->{current} || 0) + 1) % 10;
    }
    $self->{current} = $index;
    $self->{"key$index"} = $key;
    return $key;
}
